# CHIPMERGE - Combine extracted spectra from two chips into one file
# June 30, 2005
# By Doug Mink, SAO Telescope Data Center

procedure chipmerge (imagelist)

string	imagelist=""	{prompt="List of multispec file root names"}


file	keychange="hectospec$lib/chip2key"	{prompt="File of keywords to change"}
file	keyadd="hectospec$lib/chip2keys"	{prompt="File of keywords to copy"}
bool	deltemp=yes	{prompt="{Delete temporary files (y or n)"}
bool	saveold=yes	{prompt="{Save per chip spectrum files in Nomerge/ (y or n)"}
bool	verbose=no	{prompt="Print imjoin messages"}
bool	debug=no	{prompt="Print keyword transfer messages"}

begin

file	im1, im2, imout, imroot, im2t
string	keypath
file	newfile
string	iminput, tchar, templist, ims, history
int	lroot
bool	vb

	vb = verbose

#  Make temporary list of input files
	ims = imagelist
	tchar = substr (ims,1,1)
	if (tchar == '@') {
	    templist = substr (ims,2,100)
	    deltemp = no
	    }
	else {
	    templist = mktemp ("cmergelist")
	    files (ims, >templist)
	    }

	list = templist
	while (fscan (list,imin) != EOF) {
	    if (vb) {
		print ("CHIPMERGE: Input file is ", imin)
		}

#  Find root name of input files
	    smatch (imin, ".ms")
	    if (smatch.found) {
		lroot = smatch.start - 1
		smatch (imin, "_chip1.ms")
		if (smatch.found) {
		    lroot = smatch.start - 1
		    }
		smatch (imin, ".1.ms")
		if (smatch.found) {
		    lroot = smatch.start - 1
		    }
		smatch (imin, "_1.ms")
		if (smatch.found) {
		    lroot = smatch.start - 1
		    }
		smatch (imin, "_chip2.ms")
		if (smatch.found) {
		    lroot = smatch.start - 1
		    }
		smatch (imin, ".2.ms")
		if (smatch.found) {
		    lroot = smatch.start - 1
		    }
		smatch (imin, "_2.ms")
		if (smatch.found) {
		    lroot = smatch.start - 1
		    }
		imroot = substr (imin, 1, lroot)
		}
	    else {
		smatch (imin, ".fits")
		if (smatch.found) {
		    lroot = smatch.start - 1
		    imroot = substr (imin, 1, lroot)
		    }
		else {
		    imroot = imin
		    }
		}

#  If the appropriate output file has been merged already, skip it
	    imout = imroot // ".ms.fits"
	    if (access (imout)) {
		imgetp.default = "0"
		imgetp (imout, "HCMERGE")
		if (imgetp.value != "0") {
		    if (vb) {
			print ("CHIPMERGE: ", imout, " chips have been merged")
			print ("HCMERGE = ", imgetp.value)
			}
		    next
		    }
		else {
		    print ("CHIPMERGE: ", imout, " already exists; try again")
		    next
		    }
		}
	    else if (vb) {
		print ("CHIPMERGE: Root file is ", imroot)
		}

#  Create names of per chip spectrum files and check for their existence
	    im1 = imroot // "_chip1.ms.fits"
	    if (!access (im1)) {
		im1 = imroot // ".1.ms.fits"
		if (!access (im1)) {
		    im1 = imroot // "_1.ms.fits"
		    if (!access (im1)) {
			print ("CHIPMERGE: ", im1," does not exist")
			next
			}
		    }
		}
	    im2 = imroot // "_chip2.ms.fits"
	    if (!access (im2)) {
		im2 = imroot // ".2.ms.fits"
		if (!access (im2)) {
		    im2 = imroot // "_2.ms.fits"
		    if (!access (im2)) {
			print ("CHIPMERGE: ", im2," does not exist")
			next
			}
		    }
		}

#  Join multispec images from two chips into one image
	    iminput = im1 // "," // im2
	    imjoin.pixtype = ""
	    imjoin.verbose = vb
	    imjoin (iminput, imout, 2)

#  Create a temporary file to edit second chip's header
	    im2t = mktemp ("cmerge_") // "_chip2.ms.fits"
	    copy (im2, im2t)

#  Change keyword names in second chip file to start at end of first chip
	    keypath = "@" // osfn(keychange)
	    if (debug) {
		keyhead ("-v", im2t, keypath)
		}
	    else {
		keyhead (im2t, keypath)
		}

#  Add changed second chip keywords to joined file
	    keypath = "@" // osfn(keyadd)
	    if (debug) {
		chead ("-v", im2t, imout, keypath)
		}
	    else {
		chead (im2t, imout, keypath)
		}

#  Add history line including time
	    gettime ()
	    history = gettime.date // " " // gettime.time // " " // im1 // " " // im2
            hedit (imout, "HCMERGE", history, add+, verify-, show-)

#  Delete temporary file
	    if (deltemp) {
		delete (im2t)
		}

#  Save unmodified files or delete them
	    if (saveold) {
		if (!access ("Nomerge/")) {
		    mkdir ("Nomerge")
		    }
		newfile = "Notran/" // im1
		if (access (newfile)) {
		    imdelete (newfile)
		    }
		movefiles (im1, "Nomerge")
		newfile = "Notran/" // im2
		if (access (newfile)) {
		    imdelete (newfile)
		    }
		movefiles (im2, "Nomerge")
		}
	    else {
		delete (im1)
		delete (im2)
		}
	    }

#  Delete temporary list file
	if (deltemp) {
	    delete (templist)
	    }
end

# Jan 13 2005	New task
# Jun 30 2005	Change external cphead call to wcstools.chead
