# HABSKY -- Divide extracted multispec spectra by sky absorption spectra
#          using habsky1
# By Doug Mink, SAO Telescope Data Center
# May 25, 2007

procedure habsky (images)

string	images = ""	{prompt="Input multispec images to correct"}
string	skyfile="hectospec$lib/skyabs.ms.fits"	{prompt="Sky absorption spectrum normalized to continuum=1"}
bool	saveold = yes	{prompt="Save unprocessed file in Noskyab/ (y or n)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose = yes	{prompt="Log progress (y or n)"}
bool	debug = no	{prompt="Log progress (y or n)"}

begin
	bool vb, db
	file imroot, imnorm, newfile, templist, imin, imsky
	int lroot
	string imout, history, ims, tchar, keypath

	ims = images
	vb = verbose
	db = debug
	imsky = skyfile

	if (!defpac ("wcstools")) {
	    wcstools (motd=no)
	    }

	if (ims == "") {
	    print ("HABSKY:  Divide extracted multispec spectra by sky absorption spectrum")
	    print ("Usage: habsky <input images>")
	    return
	    }

#  Make temporary list of input files
	tchar = substr (ims,1,1)
        if (tchar == '@') {
	    templist = substr (ims,2,100)
	    }
	else {
	    templist = mktemp ("abskylist")
	    files (ims, >templist)
	    }
        list = templist
	habsky1.skyfile = skyfile
	habsky1.deltemp = deltemp
	habsky1.saveold = saveold
	habsky1.verbose = verbose
	habsky1.debug = debug
        while (fscan (list,imin) != EOF) {
	    habsky1 (imin)
	    }

	if (tchar != '@' && deltemp) {
	    delete (templist)
	    }
end

# Aug  6 2004	New program
# Aug 26 2004	Make it work on 1D files, too
# Aug 27 2004	Change logging keyword to HABSKY from SKYAB
# Aug 31 2004	Don't process image if HABSKY keyword is in header
# Aug 31 2004	Read next image if there is a problem instead of stopping
# Sep  2 2004	Process multiple input images
# Oct 13 2004	Check for .ms, not .ms.fit
# Oct 14 2004	Write sarith output to new file and rename it
# Oct 14 2004	Replace sarith-written APNUMnnn with APIDnnn from input file
# Oct 18 2004	Print HABSKY value if already processed

# Jan 11 2005	Improve input filename options
# Feb 23 2005	Drop keyword copying; use multispec version of sky spectrum
# Apr 14 2005	Really delete temporary files if requested
# Apr 15 2005	Fix APID keywords if they are deleted by IRAF
# Jun  8 2005	Delete APNUM keywords, copy APID keywords, allow *.ms
# Jun 30 2005	Change external cphead,delhead calls to wcstools.chead,dhead

# Oct 26 2006	Load WCSTools package if not already loaded
# Oct 31 2006	Do not delete APNUM keywords

# Feb 13 2007	Call habsky1 for all file manipulation and computation
# May 22 2007	Only check for wcstools package, not dhead task in it
# May 25 2007	Drop ap and apertures variables
