# HSPLIT - Split a single multispec hectospec file into 1-D spectra
# By Doug Mink, SAO Telescope Data Center
# October 10, 2006

procedure hsplit (msfile)

string	msfile=""	{prompt="Aperture reference multispec file (drop .ms)"}
file	keywords="hectospec$lib/keywords" {prompt="File containing header keywords to be deleted"}
bool	deltemp=yes	{prompt="Delete temporary list file (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}
bool	debug=no	{prompt="Print more information (y or n)"}

begin

	file	keyfile, keypath, imin, imroot, templist, imnew
	file	tempfile, tempfile2, outroot
	int	lroot, aperture, nfiles, i
	string	arg, tchar, ims, history, dir1d, aparg
	string	keyword, apkwr, apval1, type, x1, x2, range
	string	name, ra, dec, catnum, fiber, x, y
	string	apkey, fpkey, objkey, skykey, fibkey
	string	pra, pdec, pepoch, field, apname
	bool	vb, db

	vb = verbose
	db = debug
	imin = msfile
#	keyfile = osfn (keywords)
	keyfile = ""

#  Set defaults for scopy program
	scopy.w1 = INDEF
	scopy.w2 = INDEF
	scopy.bands = "1"
	scopy.beams = ""
	scopy.apmodulus = 0
	scopy.format = "onedspec"
	scopy.renumber = no
	scopy.offset = 0
	scopy.clobber = yes
	scopy.merge = yes
	scopy.rebin = no
	scopy.verbose = yes

	if (imin == "") {
	    print "hsplite: Make 1D spectra (fiber.object) from Hectospec multispec file"
	    return
	    }

	if (!defpac ("wcstools")) {
	    wcstools (motd=no)
	    }

# Make sure that input is a multispec file
	smatch (imin, ".ms")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imroot = substr (imin, 1, lroot)
	    smatch (imin, ".fits")
	    if (!smatch.found) {
		imnew = imin // ".fits"
		if (access (imnew)) {
		    imin = imnew
		    }
		else {
		    print ("HSPLIT: ", imin," is not a multispec file")
		    return
		    }
		}
	    }
        else {
	    smatch (imin, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imroot = substr (imin, 1, lroot)
		imnew = imroot // ".ms.fits"
		if (access (imnew)) {
		    imin = imnew
		    }
		else {
		    print ("HSPLIT: ", imin," is not a multispec file")
		    return
		    }
		}
	    else {
		imroot = imin
		imin = imroot // ".ms.fits"
		if (!access (imin)) {
		    print ("HSPLIT: ", imin," is not a multispec file")
		    return
		    }
		}
            }

# Do not split an image twice
#	imgetp.default = "0"
#	imgetp (imin, "HSPLIT")
#	if (imgetp.value != "0") {
#	    if (vb) {
#		print ("HSPLIT: ", imin, " has already been split")
#		print ("HSPLIT = ", imgetp.value)
#		}
#	    return
#	    }

	if (vb) {
	    print ("Delete keywords listed in file ",imin)
	    }
	if (vb) {
	    print ("Object name and position from ", imin)
	    }

# Make separate file to pre-edit header for output files
	tempfile = imroot // "t.ms.fits"
	if (access (tempfile)) {
	    delete (tempfile)
	    }

# Copy .fits or .imh output from DOFIBERS into file to be edited
	imcopy (imin, tempfile)

# Change pre-existing APERTURE keyword in multispec file to POINTING
        arg = tempfile // "aperture=pointing"
        keyhead (arg)

# Add history line including time
	gettime ()
	history = gettime.date // " " // gettime.time // " image split to " // "1d." // imroot
	hedit (tempfile, "HSPLIT", history, add+, verify-, show-)

# Get number of spectra in multispec file
	imgetp.default = "0"
	imgetp (imin, "i_naxis2")
	nfiles = int (imgetp.value)
	aparg = "1-" // nfiles

# Delete unneeded keywords from header (mostly APNUMnn)
	if (keyfile != "") {
	    if (vb) {
		print ("HSPLIT: delhead ",tempfile," @",keyfile)
		}
	    if (db) {
		arg = tempfile // " -v @" // keyfile
		}
	    else {
		arg = tempfile // " @" // keyfile
		}
	    dhead (arg)
	    }

# Change header position keywords
	imgetp.default = ""
	imgetp (imin, "RA")
	pra = imgetp.value
	padstr (pra)
	pra = padstr.value
	imgetp.default = ""
	imgetp (imin, "DEC")
	padstr (imgetp.value)
	pdec = padstr.value
	imgetp.default = ""
	imgetp (imin, "EQUINOX")
	imgetp.default = imgetp.value
	imgetp (imin, "EPOCH")
	pepoch = imgetp.value
	imgetp.default = ""
	imgetp (imin, "OBJECT")
	padstr (imgetp.value)
	field = padstr.value
	skykey = "field=" // field // " type=0 pra=" // pra
	skykey = skykey // " pdec=" // pdec // " pepoch=" // pepoch
	if (vb) {
	    print ("HSPLIT: ", skykey)
	    }

# Set place-holding keywords in temporary file header
#	fpkey = "epoch=2000.0 equinox=2000.0 wcsdim=1 imagetyp=OBJECT"
#	apkey = "fiber=0 aprange=0:0 fiber_x=0 fiber_y=0 catname=name catnum=0 aperture=0"
#	print "sethead ",tempfile," ",fpkey," ",skykey," ",apkey
#	arg = tempfile // " " // fpkey // " " // skykey // " " // apkey
#	sethead (arg)

# If 1d directory does not exist, create it
	dir1d = "1d." // imroot
	if (!access (dir1d)) {
	    if (vb)
		print ("HSPLIT: Making directory ", dir1d)
	    mkdir (dir1d)
	    }
	outroot = dir1d // "/temp"

# Extract all spectra into a separate FITS file
	if (vb) {
	    print ("scopy ", tempfile, " ", outroot, " apertures=", aparg)
	    }
	scopy (tempfile, outroot, apertures=aparg)

	for (i = 1; i <= nfiles; i = i + 1) {

# Get APNUM keyword value for this spectrum and extract fiber and x range
	    apkwr = "APNUM" // i
	    imgetp.default = "0"
	    imgetp (imin, apkwr)
	    if (imgetp.value == "0") {
		aperture = i
		apkey = "specnum=" // i // " aperture=" // i
		apkey = apkey // " apnum1=" // i // "_" // i
		print ("HSPLIT: No ", apkwr, " for aperture ", i)
		type = 0
		}
	    else {
		padstr (imgetp.value)
		apval1 = padstr.value

		keyword = apkwr // ",1"
		imgetp.default = "0"
		imgetp (imin, keyword)
		aperture = int (imgetp.value)
		if (db) {
		    print ("Aperture is ",aperture)
		    }

		keyword = apkwr // ",2"
		imgetp.default = "0"
		imgetp (imin, keyword)
		type = imgetp.value

		keyword = apkwr // ",3"
		imgetp.default = "0"
		imgetp (imin, keyword)
		x1 = imgetp.value

		keyword = apkwr // ",4"
		imgetp.default = x1
		imgetp (imin, keyword)
		x2 = imgetp.value
		range = x1 // ":" // x2

		apkey = "specnum=" // i // " aperture=" // aperture
		apkey = apkey // " aprange=" // range // " apnum1=" // apval1
		}

# Get pointing direction from APID keyword
	    apkwr = "APID" // i
	    imgetp.default = "0"
	    imgetp (imin, apkwr)
	    apval = imgetp.value
	    if (vb) {
		print (apkwr, " = ", apval)
		}

	    keyword = apkwr // ",1"
	    imgetp.default = "0"
	    imgetp (imin, keyword)
	    name = imgetp.value

	    keyword = apkwr // ",2"
	    imgetp.default = "0"
	    imgetp (imin, keyword)
	    padstr (imgetp.value)
	    ra = padstr.value

	    keyword = apkwr // ",3"
	    imgetp.default = "0"
	    imgetp (imin, keyword)
	    padstr (imgetp.value)
	    dec = padstr.value

	    keyword = apkwr // ",4"
	    imgetp.default = "0"
	    imgetp (imin, keyword)
	    catnum = imgetp.value

	    keyword = apkwr // ",5"
	    imgetp.default = "0"
	    imgetp (imin, keyword)
	    fiber = int (imgetp.value)

	    keyword = apkwr // ",6"
	    imgetp.default = "0"
	    imgetp (imin, keyword)
	    x = real (imgetp.value)

	    keyword = apkwr // ",7"
	    imgetp.default = "0"
	    imgetp (imin, keyword)
	    y = real (imgetp.value)

	    if (name == "target") {
		objkey = "object=" // field // "_" // catnum // " catobj=target"
		}
	    else {
		cleanstr (name)
		name = cleanstr.value
		objkey = "object=" // name // " catobj=" // name
		}
	    skykey = "pra=" // pra // " pdec=" // pdec // " pepoch=" // pepoch
	    skykey = " type=" // type // " ra=" // ra // " dec=" // dec
	    skykey = skykey // " catnum=" // catnum
	    fibkey = "fiber=" // fiber // " fiber_x=" // x
	    fibkey = fibkey // " fiber_y=" // y // " dispaxis=1"
	    if (db) {
		print (objkey, " ", apkey)
		print (skykey)
		print (fibkey)
		}

	    if (aperture < 10) {
		apname = "000" // aperture
		}
	    else if (aperture < 100) {
		apname = "00" // aperture
		}
	    else if (aperture < 1000) {
		apname = "0" // aperture
		}
	    else {
		apname = aperture
		}
	    tempfile2 = dir1d // "/temp." // apname // ".fits"
	    arg = tempfile2 // " " // fibkey // " " // objkey
	    sethead (arg)
	    if (vb) {
		print ("sethead ", arg)
		}
	    arg = tempfile2 // " " // skykey // " " // apkey
	    sethead (arg)
	    if (vb) {
		print ("sethead ", arg)
		}
	    fibarea (tempfile2, whead+, verbose=vb)

# Make up new file name from fiber number and object name
	    if (aperture < 10) {
		apname = "00" // aperture
		}
	    else if (aperture < 100) {
		apname = "0" // aperture
		}
	    else {
		apname = aperture
		}
	    if (name == "target") {
		imnew = dir1d // "/" // apname // "." // field // "_"
		imnew = imnew // catnum // ".fits"
		}
	    else {
		imnew = dir1d // "/" // apname // "." // name // "_f"
		imnew = imnew // fiber // ".fits"
		}
	    rename (tempfile2, imnew)
	    if (vb) {
		print ("rename ",tempfile2, " ", imnew)
		}
	    }

	if (deltemp) {
	    delete (tempfile)
	    }

# Add history line including time to original multispec file
	gettime ()
	history = gettime.date // " " // gettime.time // " image split to " // "1d." // imroot
	if (vb) {
	    hedit (imin, "HSPLIT", history, add+, verify-, show+)
	    }
	else {
	    hedit (imin, "HSPLIT", history, add+, verify-, show-)
	    }
	return
end

# Feb 22 2000	New program based on split2df
# May 25 2000	Add code to deal with possible input FITS file

# Oct  9 2003	Add -k option to specify keyword deletion list file
# Oct 28 2003	Update parsing code and rewrite to work on actual data
# Oct 31 2003	Save name from APIDn keyword value; use for filename if not target

# Apr 15 2004	Write APNUM1 keyword to 1D files with APNUMi value

# Mar  7 2005	Turn special characters in objet names into _
# Mar  7 2005	Set dispaxis=1
# Apr 11 2005	Run using /data/oir/bin/ksh instead of /data/mmti/bin/ksh
# Sep  8 2005	Convert to CL script from KSH script
# Sep 16 2005	Send fiberkey to correct output file

# Jul 13 2006	Use onedspec.scopy instead of imextract to split .ms files
# Aug 17 2006	Replace hsplit.cl with this
# Aug 23 2006	Write pointing FRA and FDEC as field center to FITS headers
# Sep  7 2006	Really write pointing PRA and PDEC to FITS header
# Sep  7 2006	Compute fiber area and add to FITS header as FIBAREA
# Oct 10 2006	Rename multispec file APERTURE keyword POINTING
# Oct 10 2006	Split sethead command into two pieces
