# XCPIX - Piecewise cross-correlate a spectrum against a template in pixel space
#         using MKXCTEMP and XCSAO, make new cal file id file, and reidentify
# February 10, 2004
# By Doug Mink, SAO Telescope Data Center

procedure xcpix (objects)

string	objects		{prompt="Object spectrum file"}
int	ap1=0	{prompt="First object Aperture"}
int	ap2=0	{prompt="Last object Aperture"}
file	eventemp	{prompt="Template spectrum file for even apertures"}
file	oddtemp		{prompt="Template spectrum file for odd apertures"}
file	ttemplate	{prompt="Shifted and stretched template file"}
int	firstpix=150	{prompt="Number of first pixel to correlate"}
int	lastpix=4150	{prompt="Number of last pixel to correlate"}
int	npix=200	{prompt="Number of pixels to correlate"}
int	nbin=256	{prompt="Number of rebinned pixels to correlate"}
int	intpix=100	{prompt="Interval between correlation centers in pixels"}
bool	intxcor=no	{prompt="Run cross-correlations interactively (y or n)"}
real	ndev=0		{prompt="Reject past this number of standard deviations"}
real	rmin=2.0	{prompt="Reject segment cross-correlations with R < this"}
bool	intreref=no	{prompt="Run reref fitting function interactively (y or n)"}
bool	intdisp=no	{prompt="Run reidentify function interactively (y or n)"}
bool	debug=no	{prompt="Print debugging information (y or n)"}

begin
	char tchar
	file obj1, dbfile, dbfile1
	string obj
	int icom, icom1, lcom, ap, iap1, iap2

	iap1 = ap1
	iap2 = ap2
	obj = objects
	print ("XCPIX: image ",obj," apertures ",iap1," to ",iap2)

	xcpix1.eventemp = eventemp
	xcpix1.oddtemp = oddtemp
	xcpix1.ttemplate = ttemplate
	xcpix1.firstpix = firstpix
	xcpix1.lastpix = lastpix
	xcpix1.npix = npix
	xcpix1.nbin = nbin
	xcpix1.intpix = intpix
	xcpix1.intxcor = intxcor
	xcpix1.ndev = ndev
	xcpix1.rmin = rmin
	xcpix1.intreref = intreref
	xcpix1.intdisp = intdisp
	xcpix1.debug = debug

	tchar = substr (obj,1,1)

# Process single spectrum
	if (tchar != '@') {
	    icom = stridx (obj, ',')
	    if (icom < 1) {
		if (iap1 > 0) {
		    if (iap2 > iap1) {
			for (ap = iap1; ap <= iap2; ap = ap + 1) {
			    xcpix1.objap = ap
			    xcpix1 (obj)
			    }
			}
		    else {
			print "XCPIX: single spectrum, single aperture"
		    	xcpix1.objap = iap1
			xcpix1 (obj)
			}
		    }
		else {
		    print "XCPIX: single spectrum, only aperture"
		    xcpix1 (obj)
		    }
		}

# Process comma-separated list of spectrum files
	    else {
		icom1 = 1
		while (icom > 0) {
		    lcom = icom - icom1 - 1
		    obj1 = substr (obj, icom1, lcom)
		    if (iap1 > 0) {
			if (iap2 > iap1) {
			    for (ap = iap1; ap <= iap2; ap = ap + 1) {
				xcpix1.objap = ap
				xcpix (obj1)
				}
			    }
			else {
		    	    xcpix1.objap = iap1
			    xcpix (obj1)
			    }
			}
		    else {
			xcpix1 (obj1)
			}
		    icom1 = icom + 1
		    obj = substr (obj, icom1, 128)
		    icom = stridx (obj, ',')
		    }
		obj1 = substr (obj,1, 128)
		if (iap1 > 0) {
		    if (iap2 > iap1) {
			for (ap = iap1; ap <= iap2; ap = ap + 1) {
			    xcpix1.objap = ap
			    xcpix1 (obj1)
			    }
			}
		    else {
		    	xcpix1.objap = iap1
			xcpix1 (obj1)
			}
		    }
		else {
		    xcpix1 (obj1)
		    }
		}
	    }

# Process file listing spectrum files
	else {
	    list = substr (obj,2, 128)
	    while (fscan (list, line) != EOF) {
		if (iap1 > 0) {
		    if (iap2 > iap1) {
			for (ap = iap1; ap <= iap2; ap = ap + 1) {
			    xcpix1.objap = ap
			    xcpix1 (line)
			    }
			}
		    else {
		    	xcpix1.objap = iap1
			xcpix1 (line)
			}
		    }
		else {
		    xcpix1 (line)
		    }
		}
	    }

#  Edit cross-correlated id file to match reidentified id file
        dbfile = "database/id" // obj // "x"
        dbfile1 = "database/id" // obj // "xc"
	sed ("-e s/comptemp/comp/", <dbfile, >dbfile1)

end

# Jul  1 2003	New program to call xcpix1
# Jul  9 2003	Add rmin to set minimum acceptable R value for segment xc
#
# Feb  6 2004	Instead of objap, use range objap1 to objap2
# Feb 10 2004	Instead of single template, allow two for odd and even apertures
