include	<imhdr.h>
include	<error.h>
include	<pkg/gtools.h>
include	"apertures.h"

define	APFIND		1
define	APRECENTER	2
define	APRESIZE	3
define	APEDIT		4
define	APTRACE		5
define	APSUM		6
define	APNORM		7
define	APSCAT		8
define	APALL		9
define	APFIT		10
define	APFLAT		11
define	APMASK		12
define	APSCRIPT	13
define	APSLITPROC	14
define	APNOISE		15



procedure t_apmask ()
begin
	call apall (APMASK)
end


# APALL -- Master aperture definition and extraction procedure.

procedure apall (ltask)

int	ltask			# Logical task

bool	find			# Find apertures?
bool	recenter		# Recenter apertures?
bool	resize			# Resize apertures?
bool	edit			# Edit apertures?
bool	trace			# Trace apertures?
bool	extract			# Extract apertures?
bool	fit			# Extract fit?
bool	norm			# Normalize spectra?
bool	flat			# Flatten spectra?
bool	scat			# Subtract scattered light?
bool	mask			# Aperture mask?
bool	noise			# Noise calculation?

int	input			# List of input spectra
int	refs			# List of reference spectra
int	out			# List of output spectra
pointer	format			# Output format or fit type
int	scatout			# List of scattered light images
int	profs			# List of profile spectra
int	line			# Dispersion line
int	nsum			# Lines to sum
short   maskval			# mask value

pointer	aps			# Pointer to array of aperture pointers
int	naps			# Number of apertures

char	nullstr[1]
int	i
pointer	sp, image, output, reference, profiles, str, str1

bool	clgetb(), apgetb(), streq(), ap_answer(), apgans(), apgansb()
short	clgets()
int	imtopenp(), clgeti(), ap_getim(), ap_dbaccess(), strncmp()

errchk	ap_dbacess, ap_dbread, ap_find, ap_recenter, ap_resize, ap_edit
errchk	ap_trace, ap_plot, ap_extract, ap_scatter, ap_mask, ap_dbwrite

data	nullstr /0,0/

begin
	# Allocate memory for the apertures, filenames, and strings.
	call smark (sp)
	call salloc (image, SZ_FNAME, TY_CHAR)
	call salloc (output, SZ_FNAME, TY_CHAR)
	call salloc (reference, SZ_FNAME, TY_CHAR)
	call salloc (format, SZ_LINE, TY_CHAR)
	call salloc (profiles, SZ_FNAME, TY_CHAR)
	call salloc (str, SZ_LINE, TY_CHAR)
	call salloc (str1, SZ_LINE, TY_CHAR)


	input = imtopenp ("input")
	refs = imtopenp ("references")
	line = clgeti ("line")
	nsum = clgeti ("nsum")
	out = NULL
	profs = NULL
	scatout = NULL

	out = imtopenp ("output")

	trace = false
	extract = false
	fit = false
	norm = false
	flat = false
	scat = false
	mask = false
	noise = false

	mask = clgetb ("mask")
	maskval =  clgets ("maskval")

	call ap_init (find, recenter, resize, edit, trace, extract, fit,
	        norm, flat, scat, mask, noise)

	#mask = apgans ("ansmask")

	# Initialize the apertures.
	naps = 0
	Memc[reference] = EOS
	Memc[profiles] = EOS
	call malloc (aps, 100, TY_POINTER)

	# Process the apertures from each input image.
	while (ap_getim (input, Memc[image], SZ_FNAME) != EOF) {
	    if (ap_getim (refs, Memc[str], SZ_LINE) != EOF)
		call strcpy (Memc[str], Memc[reference], SZ_FNAME)
	    if (extract || fit || flat || norm || scat || mask)
		if (ap_getim (out, Memc[output], SZ_FNAME) == EOF)
		    Memc[output] = EOS

	    # Get apertures.
	    call appstr ("ansdbwrite1", "no")
	    if (streq (Memc[reference], nullstr) ||
		streq (Memc[reference], Memc[image])) {
		if (clgetb ("verbose"))
		    call printf ("Searching aperture database ...\n")
		iferr (call ap_dbread (Memc[image], aps, naps))
		    ;
	    } else if (streq (Memc[reference], "OLD")) {
		iferr (call ap_dbread (Memc[image], aps, naps))
		    next
	    } else {
		if (strncmp (Memc[reference], "NEW", 3) == 0) {
		    if (ap_dbaccess (Memc[image]) == YES)
			next
		    call strcpy (Memc[reference+3], Memc[reference], SZ_FNAME)
		}
		if (clgetb ("verbose"))
		    call printf ("Searching aperture database ...\n")
		iferr (call ap_dbread (Memc[reference], aps, naps)) {
		    call eprintf (
			"WARNING: Reference image (%s) apertures not found\n")
			call pargstr (Memc[reference])
		    next
		}
		if (naps > 0)
		    call appstr ("ansdbwrite1", "yes")
	    }
	    call clgstr ("apertures", Memc[str], SZ_LINE)
	    call ap_select (Memc[str], Memi[aps], naps)


	    # Make a aperture mask.
	    if (mask)
		    call ap_maskn (Memc[image], Memc[output], Memi[aps], naps, maskval)


	    # Free memory.
	    for (i = 1; i <= naps; i = i + 1)
		call ap_free (Memi[aps+i-1])
	    naps = 0
	}

	# Free memory and finish up.
	call imtclose (input)
	call imtclose (refs)
	if (out != NULL)
	    call imtclose (out)
#	call ap_gclose ()
#	call ap_trfree ()
#	call apcpset ()
	call sfree (sp)
end


procedure ap_init (find, recenter, resize, edit, trace, extract, fit,
	norm, flat, scat, mask, noise)

bool	find, recenter, resize, edit, trace
bool	extract, fit, norm, flat, scat, mask, noise

pointer	sp, str
bool	clgetb()

begin
	call smark (sp)
	call salloc (str, SZ_LINE, TY_CHAR)

	if (mask)
	    call appans ("ansmask", mask, mask)

	call sfree (sp)
end
